@@ -1,5 +1,5 @@
 /* Lattice Mico32-specific support for 32-bit ELF
-   Copyright 2008   Free Software Foundation, Inc.
+   Copyright 2008, 2009  Free Software Foundation, Inc.
    Contributed by Jon Beniston <jon@beniston.com>
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -225,20 +225,10 @@ create_got_section (bfd *dynobj, struct bfd_link_info *info)
   htab = lm32_elf_hash_table (info);
   htab->sgot = bfd_get_section_by_name (dynobj, ".got");
   htab->sgotplt = bfd_get_section_by_name (dynobj, ".got.plt");
-  if (! htab->sgot || ! htab->sgotplt)
+  htab->srelgot = bfd_get_section_by_name (dynobj, ".rela.got");
+  if (! htab->sgot || ! htab->sgotplt || ! htab->srelgot)
     abort ();
 
-  htab->srelgot = bfd_make_section_with_flags (dynobj, ".rela.got",
-					       (SEC_ALLOC
-						| SEC_LOAD
-						| SEC_HAS_CONTENTS
-						| SEC_IN_MEMORY
-						| SEC_LINKER_CREATED
-						| SEC_READONLY));
-  if (htab->srelgot == NULL
-      || ! bfd_set_section_alignment (dynobj, htab->srelgot, 2))
-    return FALSE;
-
   return TRUE;
 }
 
@@ -1140,10 +1130,8 @@ lm32_elf_relocate_section (bfd *output_bfd,
 
       if (r != bfd_reloc_ok)
         {
-          const char *name;
           const char *msg = NULL;
           arelent bfd_reloc;
-          reloc_howto_type *howto;
 
           lm32_info_to_howto_rela (input_bfd, &bfd_reloc, rel);
           howto = bfd_reloc.howto;
