@@ -390,7 +390,7 @@ enum SHT
 
   // Link editor is to sort the entries in this section based on the
   // address specified in the associated symbol table entry.
-  SHT_ORDERED = 0x7fffffff,
+  SHT_ORDERED = 0x7fffffff
 };
 
 // The valid bit flags found in the Shdr sh_flags field.
@@ -514,7 +514,7 @@ enum STT
 
   // ARM: a THUMB function.  This is not defined in ARM ELF Specification but
   // used by the GNU tool-chain.
-  STT_ARM_TFUNC = 13,
+  STT_ARM_TFUNC = 13
 };
 
 inline STB
@@ -656,7 +656,11 @@ enum DT
   DT_FINI_ARRAYSZ = 28,
   DT_RUNPATH = 29,
   DT_FLAGS = 30,
+
+  // This is used to mark a range of dynamic tags.  It is not really
+  // a tag value.
   DT_ENCODING = 32,
+
   DT_PREINIT_ARRAY = 32,
   DT_PREINIT_ARRAYSZ = 33,
   DT_LOOS = 0x6000000d,
@@ -751,7 +755,7 @@ enum DF_1
   DF_1_INTERPOSE = 0x400,
   DF_1_NODEFLIB = 0x800,
   DF_1_NODUMP = 0x1000,
-  DF_1_CONLFAT = 0x2000,
+  DF_1_CONLFAT = 0x2000
 };
 
 // Version numbers which appear in the vd_version field of a Verdef
