@@ -23,7 +23,6 @@
 #define OBJ_HEADER "obj-coff.h"
 
 #include "as.h"
-#include "safe-ctype.h"
 #include "obstack.h"
 #include "subsegs.h"
 
@@ -622,14 +621,11 @@ obj_coff_def (int what ATTRIBUTE_UNUSED)
   demand_empty_rest_of_line ();
 }
 
-unsigned int dim_index;
-
 static void
 obj_coff_endef (int ignore ATTRIBUTE_UNUSED)
 {
   symbolS *symbolP = NULL;
 
-  dim_index = 0;
   if (def_symbol_in_progress == NULL)
     {
       as_warn (_(".endef pseudo-op used outside of .def/.endef: ignored."));
@@ -862,7 +858,7 @@ obj_coff_endef (int ignore ATTRIBUTE_UNUSED)
 static void
 obj_coff_dim (int ignore ATTRIBUTE_UNUSED)
 {
-  int dim_index;
+  int d_index;
 
   if (def_symbol_in_progress == NULL)
     {
@@ -873,10 +869,10 @@ obj_coff_dim (int ignore ATTRIBUTE_UNUSED)
 
   S_SET_NUMBER_AUXILIARY (def_symbol_in_progress, 1);
 
-  for (dim_index = 0; dim_index < DIMNUM; dim_index++)
+  for (d_index = 0; d_index < DIMNUM; d_index++)
     {
       SKIP_WHITESPACES ();
-      SA_SET_SYM_DIMEN (def_symbol_in_progress, dim_index,
+      SA_SET_SYM_DIMEN (def_symbol_in_progress, d_index,
 			get_absolute_expression ());
 
       switch (*input_line_pointer)
@@ -890,7 +886,7 @@ obj_coff_dim (int ignore ATTRIBUTE_UNUSED)
 	  /* Fall through.  */
 	case '\n':
 	case ';':
-	  dim_index = DIMNUM;
+	  d_index = DIMNUM;
 	  break;
 	}
     }
@@ -1544,7 +1540,6 @@ coff_frob_file_after_relocs (void)
   						 'r' for read-only data
   						 's' for shared data (PE)
 						 'y' for noread
-					   '0' - '9' for power-of-two alignment (GNU extension).
    But if the argument is not a quoted string, treat it as a
    subsegment number.
 
@@ -1557,7 +1552,6 @@ obj_coff_section (int ignore ATTRIBUTE_UNUSED)
   /* Strip out the section name.  */
   char *section_name;
   char c;
-  int alignment = -1;
   char *name;
   unsigned int exp;
   flagword flags, oldflags;
@@ -1600,11 +1594,6 @@ obj_coff_section (int ignore ATTRIBUTE_UNUSED)
 		 attr != '"'
 		 && ! is_end_of_line[attr])
 	    {
-	      if (ISDIGIT (attr))
-		{
-		  alignment = attr - '0';
-		  continue;
-		}
 	      switch (attr)
 		{
 		case 'b':
@@ -1681,8 +1670,6 @@ obj_coff_section (int ignore ATTRIBUTE_UNUSED)
     }
 
   sec = subseg_new (name, (subsegT) exp);
-  if (alignment >= 0)
-    sec->alignment_power = alignment;
 
   oldflags = bfd_get_section_flags (stdoutput, sec);
   if (oldflags == SEC_NO_FLAGS)
@@ -1940,5 +1927,6 @@ const struct format_ops coff_format_ops =
   coff_pop_insert,
   0,	/* ecoff_set_ext */
   coff_obj_read_begin_hook,
-  coff_obj_symbol_new_hook
+  coff_obj_symbol_new_hook,
+  coff_obj_symbol_clone_hook
 };
