@@ -150,8 +150,7 @@ fix_new_internal (fragS *frag,		/* Which frag?  */
 		  symbolS *sub_symbol,	/* X_op_symbol.  */
 		  offsetT offset,	/* X_add_number.  */
 		  int pcrel,		/* TRUE if PC-relative relocation.  */
-		  RELOC_ENUM r_type ATTRIBUTE_UNUSED /* Relocation type.  */,
-		  int at_beginning)	/* Add to the start of the list?  */
+		  RELOC_ENUM r_type ATTRIBUTE_UNUSED /* Relocation type.  */)
 {
   fixS *fixP;
 
@@ -195,6 +194,10 @@ fix_new_internal (fragS *frag,		/* Which frag?  */
 
   as_where (&fixP->fx_file, &fixP->fx_line);
 
+  /* Usually, we want relocs sorted numerically, but while
+     comparing to older versions of gas that have relocs
+     reverse sorted, it is convenient to have this compile
+     time option.  xoxorich.  */
   {
 
     fixS **seg_fix_rootP = (frags_chained
@@ -204,22 +207,22 @@ fix_new_internal (fragS *frag,		/* Which frag?  */
 			    ? &seg_info (now_seg)->fix_tail
 			    : &frchain_now->fix_tail);
 
-    if (at_beginning)
-      {
-	fixP->fx_next = *seg_fix_rootP;
-	*seg_fix_rootP = fixP;
-	if (fixP->fx_next == NULL)
-	  *seg_fix_tailP = fixP;
-      }
+#ifdef REVERSE_SORT_RELOCS
+
+    fixP->fx_next = *seg_fix_rootP;
+    *seg_fix_rootP = fixP;
+
+#else /* REVERSE_SORT_RELOCS  */
+
+    fixP->fx_next = NULL;
+
+    if (*seg_fix_tailP)
+      (*seg_fix_tailP)->fx_next = fixP;
     else
-      {
-	fixP->fx_next = NULL;
-	if (*seg_fix_tailP)
-	  (*seg_fix_tailP)->fx_next = fixP;
-	else
-	  *seg_fix_rootP = fixP;
-	*seg_fix_tailP = fixP;
-      }
+      *seg_fix_rootP = fixP;
+    *seg_fix_tailP = fixP;
+
+#endif /* REVERSE_SORT_RELOCS  */
   }
 
   return fixP;
@@ -237,7 +240,7 @@ fix_new (fragS *frag,		/* Which frag?  */
 	 RELOC_ENUM r_type		/* Relocation type.  */)
 {
   return fix_new_internal (frag, where, size, add_symbol,
-			   (symbolS *) NULL, offset, pcrel, r_type, FALSE);
+			   (symbolS *) NULL, offset, pcrel, r_type);
 }
 
 /* Create a fixup for an expression.  Currently we only support fixups
@@ -305,19 +308,7 @@ fix_new_exp (fragS *frag,		/* Which frag?  */
       break;
     }
 
-  return fix_new_internal (frag, where, size, add, sub, off, pcrel,
-			   r_type, FALSE);
-}
-
-/* Create a fixup at the beginning of FRAG.  The arguments are the same
-   as for fix_new, except that WHERE is implicitly 0.  */
-
-fixS *
-fix_at_start (fragS *frag, int size, symbolS *add_symbol,
-	      offsetT offset, int pcrel, RELOC_ENUM r_type)
-{
-  return fix_new_internal (frag, 0, size, add_symbol,
-			   (symbolS *) NULL, offset, pcrel, r_type, TRUE);
+  return fix_new_internal (frag, where, size, add, sub, off, pcrel, r_type);
 }
 
 /* Generic function to determine whether a fixup requires a relocation.  */
@@ -1796,22 +1787,24 @@ write_object_file (void)
 	  if (symbol_equated_reloc_p (symp)
 	      || S_IS_WEAKREFR (symp))
 	    {
-	      const char *name = S_GET_NAME (symp);
+	      const char *sname = S_GET_NAME (symp);
+
 	      if (S_IS_COMMON (symp)
-		  && !TC_FAKE_LABEL (name)
+		  && !TC_FAKE_LABEL (sname)
 		  && !S_IS_WEAKREFR (symp)
 		  && (!S_IS_EXTERNAL (symp) || S_IS_LOCAL (symp)))
 		{
 		  expressionS *e = symbol_get_value_expression (symp);
+
 		  as_bad (_("Local symbol `%s' can't be equated to common symbol `%s'"),
-			  name, S_GET_NAME (e->X_add_symbol));
+			  sname, S_GET_NAME (e->X_add_symbol));
 		}
 	      if (S_GET_SEGMENT (symp) == reg_section)
 		{
 		  /* Report error only if we know the symbol name.  */
 		  if (S_GET_NAME (symp) != reg_section->name)
 		    as_bad (_("can't make global register symbol `%s'"),
-			    name);
+			    sname);
 		}
 	      symbol_remove (symp, &symbol_rootP, &symbol_lastP);
 	      continue;
